ttest <- function(y1, y2, test = "two-sided", alpha = 0.05)
{
  n1 <- length(y1); n2 <- length(y2)
  ndf <- n1 + n2 - 2
  s2 <- ((n1 - 1) * var(y1) + (n2 - 1) * var(y2))/ndf
  tstat <- (mean(y1) - mean(y2))/sqrt(s2 * (1/n1 + 1/n2))
  tail.area <- switch(test,
    "two-sided" = 2 * (1 - pt(abs(tstat), ndf)),
    lower = pt(tstat, ndf),
    upper = 1 - pt(tstat, ndf),
    {
      warning("test must be 'two-sided', 'lower' or 'upper'")
      NULL
    }
  )
  list(tstat = tstat, df = ndf,
       reject = if(!is.null(tail.area)) tail.area < alpha,
       tail.area = tail.area)
}

ttest <- function(y1, y2,
  test = c("two-sided", "lower", "upper"), alpha = 0.05)
{
  test <- match.arg(test)
  n1 <- length(y1); n2 <- length(y2)
  ndf <- n1 + n2 - 2
  s2 <- ((n1 - 1) * var(y1) + (n2 - 1) * var(y2))/ndf
  tstat <- (mean(y1) - mean(y2))/sqrt(s2 * (1/n1 + 1/n2))
  tail.area <- switch(test,
    "two-sided" = 2 * (1 - pt(abs(tstat), ndf)),
    lower = pt(tstat, ndf),
    upper = 1 - pt(tstat, ndf),
    {
      warning("test must be 'two-sided', 'lower' or 'upper'")
      NULL
    }
  )
  list(tstat = tstat, df = ndf,
       reject = if(!is.null(tail.area)) tail.area < alpha,
       tail.area = tail.area)
}
