Ttest <- function(z, ...) {
  if(is.null(class(z))) class(z) <- data.class(z)
  UseMethod("Ttest") # UseMethod("Ttest", z, ...) in R
}

Ttest.default <- function(z, y2, ..., alpha = 1/20,
         test = c("two-sided", "lower tailed", "upper tailed"))
{
  n1 <- length(z)
  n2 <- length(y2)
  ndf <- n1 + n2 - 2
  s2 <- ((n1 - 1) * var(z) + (n2 - 1) * var(y2))/ndf
  tstat <- (mean(z) - mean(y2))/sqrt(s2 * (1/n1 + 1/n2))
  tail.area <- switch(test <- match.arg(test),
    "two-sided" = 2 * (1 - pt(abs(tstat), ndf)),
    "lower tailed" = pt(tstat, ndf),
    "upper tailed" = 1 - pt(tstat, ndf),
    {
      warning("test must be 'two-sided', 'lower' or 'upper'")
      NULL
    })
   res <- list("t-stat"=tstat, d.f.=ndf, y1=z, y2=y2,
               test=test, tail.area=tail.area,
               reject=tail.area < alpha, alpha=alpha)
   class(res) <- "my.t.test"
   res
}

print.my.t.test <- function(x, ...) {
  print(unlist(x[1:2]))
  invisible(x)
}

summary.my.t.test <- function(object)
  structure(object, class = c("sum.my.t.test", class(object)))

print.sum.my.t.test <- function(x, ...) {
  n1 <- length(x$y1)
  n2 <- length(x$y2)
  cat("\n T-test of  samples: \n")
  cat("Sample 1:", format(x$y1[1:min(n1, 5)], ...))
  cat(if(n1 <= 5) "" else " ...", "\n")
  cat("Sample 2:", format(x$y2[1:min(n2, 5)], ...))
  cat(if(n2 <= 5) "" else " ...", "\n")
  cat("\nt =", format(x$"t-stat", ...), "on",
      x$d.f., "d.f.\n")
  cat("Test:", x$test, "\n")
  cat("Tail area:", format(x$tail.area, ...), "\n")
  cat("Level:", format(x$alpha, ...), "\t")
  cat("Null hypothesis",
      if(x$reject) "rejected." else "retained.", "\n")
  invisible(x)
}

plot.my.t.test <- function(x, ...)
    invisible(boxplot(x[c("y1", "y2")], ...))

Ttest.matrix <- function(z, ...) Ttest(z[, 1], z[, 2], ...)
Ttest.list <- function(z, ...) Ttest(z[[1]], z[[2]], ...)
Ttest.factor <- function(z, y, ...) {
   lev <- levels(z)
   Ttest(y[z == lev[1]], y[z == lev[2]])
}
