.First.lib <- function(lib, pkg)
  library.dynam("Polynomial", pkg, lib)

.poly.mult <- function(e1, e2) {
  storage.mode(e1) <- "double"
  storage.mode(e2) <- "double"
  le1 <- length(e1)
  le2 <- length(e2)
  .C("VR_polymul",
     e1, as.integer(le1), e2, as.integer(le2),
     e1.e2 = double(le1 + le2 - 1))$e1.e2
}

.poly.quo.rem <- function(e1, e2) {
  l1 <- length(e1)
  l2 <- length(e2)
  if(l1 < l2)
    return(list(quotient = 0, remainder = e1))
  if (l2 == 1)
    return(list(quotient = e1/e2, remainder = 0))
  storage.mode(e1) <- "double"
  storage.mode(e2) <- "double"
  .C("VR_polydiv", remainder = e1, as.integer(l1), e2, as.integer(l2),
     quotient = double(l1 - l2 + 1))[c("quotient", "remainder")]
}

Math.polynomial <- function (x, digits, ...) {
  x <- as.numeric(x)
  switch(.Generic,
         round = , signif = polynomial(NextMethod(.Generic)),
         {
           warning("polynomial coerced to numeric vector")
           NextMethod(.Generic)
         })
}

Ops.polynomial <- function (e1, e2) {
  if (missing(e2)) {
    ## Unary operators
    e1 <- unclass(e1)
    return(polynomial(NextMethod(.Generic)))
  }
  e1 <- unclass(e1)
  e2 <- unclass(e2)
  e1.op.e2 <-
    switch(.Generic,
           "+" = , "-" = {
             if((l1 <- length(e1)) < (l2 <- length(e2)))
               e1 <- c(e1, rep(0, l2 - l1))
             if(l2 < l1)
               e2 <- c(e2, rep(0, l1 - l2))
             NextMethod(.Generic)
           },
           "*" = .poly.mult(e1, e2),
           "/" = , "%/%" = .poly.quo.rem(e1, e2)$quotient,
           "%%" = .poly.quo.rem(e1, e2)$remainder,
           "^" = if (length(e2) > 1 || e2 < 0 || (e2 %% 1) != 0)
           stop("non positive integer power requested")
           else {
             switch(as.character(e2),
                    "0" = polynomial(1),
                    "1" = polynomial(e1),
                    {
                      m <- e1
                      for (i in 2:e2)
                        m <- .poly.mult(m, e1)
                      m
                    })
           },
           "==" = length(e1) == length(e2) && all(e1 == e2),
           "!=" = length(e1) != length(e2) || any(e1 != e2),

           stop(paste("\"", .Generic, "\"",
                      " is undefined for polynomials",
                      sep = "")))
  polynomial(e1.op.e2)
}

Summary.polynomial <- function (x, ...) {
  warning("polynomial coerced to numeric vector")
  x <- as.numeric(x)
  NextMethod(.Generic)
}

"[.polynomial" <- function (x, i, ...) {
  x <- unclass(x)
  polynomial(NextMethod("["))
}

"[<-.polynomial" <- function (x, i, value) {
  value <- as.numeric(value)
  x <- unclass(x)
  x[i] <- value
  polynomial(x)
}

makePolyChar <- function(p, rational = FALSE) {
  p <- as.numeric(p)
  tap <- if(rational) .vfrac(abs(p)) else
           as.character(zapsmall(abs(p)))
  lp <- length(p) - 1
  names(p) <- 0:lp
  if(any(discard <- match(tap, c("0", "-0"),
                       nomatch = FALSE))) {
    p <- p[!discard]
    if(length(p) == 0)
      return("0")
    tap <- tap[!discard]
  }
  sp <- c("- ", "+ ")[(p > 0) + 1]
  np <- names(p)
  p <- tap
  p[p == "1" & np != "0"] <- ""
  pow <- paste("x", "^", np, sep = "")
  pow[np == "0"] <- ""
  pow[np == "1"] <- "x"
  stars <- rep("*", length(p))
  stars[p == "" | pow == ""] <- ""
  p <- paste(sp, p, stars, pow, sep = "", collapse = " ")
  while((ch <- substring(p, 1, 1)) == "+" || ch == " ")
    p <- substring(p, 2)
  p
}

as.character.polynomial <- function(x)
  makePolyChar(unclass(x))

as.function.polynomial <- function (a) {
  p <- as.name("p")
  x <- as.name("x")
  a <- unclass(a)
  if ((an <- length(a)) == 1) {
    a <- c(a, 0)
    an <- 2
  }
  statement <-
    call("{", call("<-", p, call("+", a[an-1], call("*", x, a[an]))))
  i <- length(statement)
  for (ai in rev(a)[-(1:2)])
    statement[[i <- i + 1]] <-
      call("<-", p, call("+", ai, call("*", x, p)))
  statement[[i]] <- statement[[i]][[3]]
  if(an == 2) statement <- statement[[2]]
  fun <- function(x) {}
  body(fun, envir = NULL) <- statement
  fun
}

makePolyFunction <- function (x, ...) {
  ## uses `closure' and non-S-style visibility
  x <- unclass(x)
  function(v) {
    p <- 0
    for(ai in rev(x))
      p <- ai + v * p
    p
  }
}

as.polynomial <- function (p)
  if (is.polynomial(p)) p else polynomial(p)

polynomial <- function(a = c(0, 1)) {
#  if(inherits(a, "polynomial")) return(a)
  if(!(is.numeric(a) || is.complex(a))) {
    warning("non-numeric coefficients")
    a <- as.numeric(a)
  }
  if(any(!is.finite(a))) stop("invalid coefficient vector")
  la <- la0 <- length(a)
  if(la == 0)
    a <- 0
  else {
    while (a[la] == 0 && la > 1)
      la <- la - 1
    if(la < la0) a <- a[1:la]
  }
  class(a) <- "polynomial"
  a
}

is.polynomial <- function(p) inherits(p, "polynomial")

coef.polynomial <- function (object, ...) as.vector(object)

deriv.polynomial <- function (x, ...) {
  x <- unclass(x)
  if (length(x) == 1)
    return(polynomial(0))
  x <- x[-1]
  polynomial(x * seq(along = x))
}

integral <- function (expr, ...)
UseMethod("integral")

integral.default <- function (x, ...)
stop(paste("no integral method defined for objects of class",
    data.class(x)))

integral.polynomial <- function (expr, limits = NULL, ...) {
  expr <- unclass(expr)
  p <- polynomial(c(0, expr/seq(along = expr)))
  if (is.null(limits))
    p
  else diff(predict(p, limits))
}

lines.polynomial <- function (x, len = 300, ...) {
  p <- x
  pu <- par("usr")
  x <- seq(pu[1], pu[2], len = len)
  y <- predict(p, x)
  y[y <= pu[3] | y >= pu[4]] <- NA
  lines(x, y, ...)
}

plot.polynomial <- function (x, xlim = 0:1, ylim = range(Px),
                             type = "l", len = 300, ...) {
  p <- x
  if (missing(xlim))
    xlim <- range(Re(unlist(summary(p))))
  if (any(is.na(xlim))) {
    warning("summary of polynomial fails. Using nominal xlim")
    xlim <- 0:1
  }
  if (diff(xlim) == 0)
    xlim <- xlim + c(-1, 1)/2
  if (length(xlim) > 2) ### gives the points precisely
    x <- xlim
  else {
    eps <- diff(xlim)/100
    xlim <- xlim + c(-eps, eps)
    x <- seq(xlim[1], xlim[2], len = len)
  }
  Px <- predict(p, x)
  if (!missing(ylim))
    Px[Px < ylim[1]] <- Px[Px > ylim[2]] <- NA
  plot(x, Px, type = type, xlim = xlim, ylim = ylim, ...)
}

points.polynomial <- function (x, len = 300, ...) {
  p <- x
  pu <- par("usr")
  x <- seq(pu[1], pu[2], length = len)
  y <- predict(p, x)
  out <- y <= pu[3] | y >= pu[4]
  y[out] <- NA
  points(x, y, ...)
}

calcCoef <-
function (x, y, tol = sqrt(.Machine$double.eps)) {
  if (missing(y)) {  ## roots prescribed, find monic poly.
    p <- 1
    for (xi in x) p <- c(0, p) - c(xi * p, 0)
    return(polynomial(p))
  }
  ### x and y given: Lagrange interpolation
  if ((m <- length(x)) != length(y))
    stop("x and y(x) do not match in length!")
  if (any(discard <- duplicated(x))) {
    crit <- max(tapply(y, x, function(x) diff(range(x))))
    if (crit > tol)
      warning("some duplicated x-points have inconsistent y-values")
    keep <- !discard
    y <- y[keep]
    x <- x[keep]
  }
  if (m <= 1)
    return(polynomial(y))
  r <- 0
  for (i in 1:m)
    r <- r + (y[i] * unclass(Recall(x[-i])))/prod(x[i] - x[-i])
  r[abs(r) < tol] <- 0
  r
}

polyWithZeros <- polyWithRoots <- function(x) polynomial(calcCoef(x))
polyInterp <- function(x, y) polynomial(calcCoef(x, y))

predict.polynomial <- function (object, newdata, ...) {
  v <- 0
  object <- rev(unclass(object))
  for (pj in object) v <- newdata * v + pj
  v
}

print.polynomial <- function (x, ..., rational = FALSE) {
  ow <- max(35, unlist(options("width")) - 1)
  p <- makePolyChar(unclass(x), rational = rational)
  m2 <- 0
  np <- nchar(p)
  while (m2 < np) {
    m1 <- m2 + 1
    m2 <- m2 + ow
    if (m2 < np)
      while (m2 > m1 && substring(p, m2, m2) != " ")
        m2 <- m2 - 1
    cat(substring(p, m1, m2), "\n")
  }
  invisible(x)
}

print.summary.polynomial <- function (x, ...) {
  cat("\n Summary information for:\n")
  print(attr(x, "originalPolynomial"))
  cat("\n Zeros:\n")
  print(x$zeros)
  cat("\n Stationary points:\n")
  print(x$stationaryPoints)
  cat("\n Points of inflexion:\n")
  print(x$inflexionPoints)
  invisible(x)
}

solve.polynomial <- function (a, b, ...) {
  ## intended to solve a(x) = 0 for x
  ## (or a(x) = b(x) if b is not missing).
  if (!missing(b))
    a <- a - b
  a <- unclass(a)
  r <- numeric(0)
  z <- 0
  while(a[z + 1] == 0) z <- z+1  # factor out zero roots
  if(z > 0) {
    a <- a[-(1:z)]
    r <- rep(0, z)
  }
  switch(as.character(length(a)),
         "0" = ,
         "1" = r,
         "2" = sort(c(r, -a[1]/a[2])),
         sort(c(r, polyroot(a))))
}

summary.polynomial <- function (object) {
  dp <- deriv(object)
  structure(list(zeros = solve(object),
                 stationaryPoints = solve(dp),
                 inflexionPoints = solve(deriv(dp))),
            class = "summary.polynomial",
            originalPolynomial = object)
}

.vfrac <- function(x, ...) {
  ans <- .rat(x, ...)
  ndc <- paste(ans$rat[, 1], ans$rat[, 2], sep = "/")
  int <- ans$rat[, 2] == 1
  ndc[int] <- as.character(ans$rat[int, 1])
  ndc
}

.rat <- function(x, cycles = 10, max.denominator = 2000) {
  a0 <- rep(0, length(x))
  A <- matrix(b0 <- rep(1, length(x)))
  B <- matrix(floor(x))
  r <- as.vector(x) - drop(B)
  len <- 0
  while(any(which <- (r > 1/max.denominator)) &&
        (len <- len + 1) <= cycles) {
    a <- a0
    b <- b0
    a[which] <- 1
    r[which] <- 1/r[which]
    b[which] <- floor(r[which])
    r[which] <- r[which] - b[which]
    A <- cbind(A, a)
    B <- cbind(B, b)
  }
  pq1 <- cbind(b0, a0)
  pq <- cbind(B[, 1], b0)
  len <- 1
  while((len <- len + 1) <= ncol(B)) {
    pq0 <- pq1
    pq1 <- pq
    pq <- B[, len] * pq1 + A[, len] * pq0
  }
  list(rat = pq, x = x)
}

poly.orth <- function(x, degree = length(unique(x)) - 1, norm = TRUE)
{
  at <- attr(poly(x, degree), "coefs")
  a <- at$alpha
  N <- at$norm2
  x <- polynomial()
  p <- list(polynomial(0), polynomial(1))
  for(j in 1:degree)
    p[[j + 2]] <- (x - a[j]) * p[[j + 1]] - N[j + 1]/N[j] * p[[j]]
  p <- p[-1]
  if(norm) {
    sqrtN <- sqrt(N[-1])
    for(j in 1 + 0:degree) p[[j]] <- p[[j]]/sqrtN[j]
  }
  p
}
